function [dctf,Lm]=contras3(s,Ncuan,norm,To,DUmax,DVmax)

% CONTRAS3 pasa a contrastes las amplitudes de las DCTs locales true-color de 
% resolucion Ncuan de una imagen YUV 
%
% Una resolucion Ncuan en una imagen de lado L_im genera bloques de lado L_b = L/(2^Ncuan);
%
% La definicion de la DCT 2D discreta de una imagen de tamao M*N implica que:
%                                
%    * Valor Medio:       
%                                1
%                        To = -------- DCT(0,0)
%                              sqrt(M*N)
%
%    * Desviacion respecto de la media con frecuencia correspondiente a la
%      funcion base i,j es:
%
%            Delta T ij = alfa(i)*beta(j) * DCT(i,j)
%
%            con:
%                            /
%                           |    1/sqrt(M)
%                 alfa(i) = <
%                           |
%                            \   sqrt(2)/sqrt(M)
%
%                            /
%                           |    1/sqrt(N)
%                 beta(j) = <
%                           |
%                            \   sqrt(2)/sqrt(N)
%
% Normalizando por un color con luminancia To, el contraste ACROMATICO de 
% cada componente i,j sera:
%
%                C(i,j) = alfa(i)*beta(j)*DCT(i,j)/To
% 
% Si el color To es el color promedio del bloque, tenemos:
%
%                C(i,j) = sqrt(M*N)*alfa(i)*beta(j)*DCT(i,j)/DCT(0,0)
%
% Los contrastes CROMATICOS (ver notas de la patente) se calculan comparando
% las modulaciones en cada direccion Delta Tij (que se obtienen de la
% transformada segun las expresiones anteriores) con la modulacion maxima que 
% (sobre un conjunto de imagenes) da lugar a unos colores extremos generables. 
% Esos colores extremos determinan la pareja de redes de luminancia con las que 
% construir redes puramente cromaticas. El contraste de estas redes de luminancia 
% de los colores extremos es el contraste cromatico.
% 
% Pruebas empiricas sobre un conjunto de 19 imagenes en color (asumiendo que sus
% niveles digitales [0,255] son valores triestimulo RGB NTSC) sugieren que las 
% modulaciones maximas para tener colores extremos generables alrededor del color 
% promedio de dichas imagenes es DeltaTmax=40 (en U y V).
% Este valor es tan arbitrario o tan poco arbitrario como decir que la
% luminancia media de las imagenes esta en torno a 128 y que la modulacion
% de luminancia maxima en imagenes va a ser 128.
% No obstante, dejo esto abierto para jugar (si escau).
%
% CONTRAS3 permite tres tipos de normalizacion:
%
%  * Utilizando el color promedio global de la imagen                 (norm=1)
%  * Utilizando el color promedio de cada bloque                      (norm=2) 
%  * Utilizando como referencia el color facilitado To (en RGB NTSC)  (norm=3)
%    (si no se va a usar esta tercera opcion, poner To=0).
%    Como referencia, el color medio de las 19 imagenes analizadas (en RGB NTSC 
%    sin calibrar) es To=[132 116 90]  (que tiene una luminancia de 118 cd/m2)
%
% Si la transformada que se le pasa tiene una sola capa, se asume que la
% cosa es acromatica y solo se calcula el contraste acromatico.
%
% NOTA: solo funciona para imagenes cuadradas de lado potencia de 2
%
% La inversa de CONTRAS3 es ICONTRAS3
% 
% USO: [dctf,Lm]=contras3(dcti,Ncuan,Norm,To,DUmax,DVmax);
%
   

% Inicializacion de parametros %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   ss=s;
   tam=size(ss);
   
   if length(tam)==3
      capas=tam(3);
   else
      capas=1;
   end    
   tam=tam(1);
   
   lados=[tam tam];

   dctf=zeros(tam,tam,capas);
   
   lcuan=tam/(2^Ncuan);

   posai=[tam tam]/2-round(tam/2);
   posbd=[tam tam]/2+round(tam/2);
   coorcuanai=floor([(posai(1)-1)/lcuan+1 (posai(2)-1)/lcuan+1]);
   coorcuanbd=floor([(posbd(1)-1)/lcuan+1 (posbd(2)-1)/lcuan+1]);
   if coorcuanai(1)<1
        coorcuanai(1)=1;
   end 
   if coorcuanai(2)<1
        coorcuanai(2)=1;
   end 
   if coorcuanbd(1)>(2^Ncuan)
        coorcuanbd(1)=2^Ncuan;
   end     
   if coorcuanai(2)>(2^Ncuan)
        coorcuanbd(2)=2^Ncuan;
   end 

% Transformaciones cromaticas %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
   
   Mrgb_yuv =[0.2990    0.5870    0.1140;-0.1726   -0.3388    0.5114;0.5114   -0.4282   -0.0832];
   Myuv_rgb = inv(Mrgb_yuv);

% Promedio %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
   
if norm==1
    DCT11=[0 0 0];
    Nbloques=0;
    for ii=coorcuanai(1):coorcuanbd(1)
        for jj=coorcuanai(2):coorcuanbd(2)
            
            iii=(ii-1)*lcuan+ceil(lcuan/2);
            jjj=(jj-1)*lcuan+ceil(lcuan/2); 
            
            for i=1:capas
                s=ss(:,:,i);
                blo=sacasub(s,[iii jjj],[lcuan lcuan],0);
                DCT11(i)=DCT11(i)+blo(1,1);
            end
            Nbloques=Nbloques+1;  
            
        end 
    end
    DCT11=DCT11/Nbloques;
    if capas==3
       To_yuv=DCT11/lcuan;
       To=(Myuv_rgb*To_yuv')';
    else
       To=DCT11/lcuan;
       To_yuv=To;
    end
elseif norm==2
    To=0; 
else
    if capas==3
       To_yuv=(Mrgb_yuv*To')';
    else
       To_yuv=To;
    end
end    

% Expresion de cada bloque en contraste %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

factores_DCT_discreta=2*ones(lcuan,lcuan)/lcuan;
factores_DCT_discreta(1,:)=sqrt(2)*ones(1,lcuan)/lcuan;
factores_DCT_discreta(:,1)=sqrt(2)*ones(lcuan,1)/lcuan;
factores_DCT_discreta(1,1)=1/lcuan;

dctf=ss;

for i=1:capas
    
    s=ss(:,:,i);
    for ii=coorcuanai(1):coorcuanbd(1)
        for jj=coorcuanai(2):coorcuanbd(2)
            
            iii=(ii-1)*lcuan+ceil(lcuan/2);
            jjj=(jj-1)*lcuan+ceil(lcuan/2); 
            blo=sacasub(s,[iii jjj],[lcuan lcuan],0);
            
            if i==1
                if norm==1
                    deltaT=factores_DCT_discreta.*blo;
                    contraste=deltaT/To_yuv(1);
                    Lm(ii,jj,i)=deltaT(1,1);
                    contraste(1,1)=0;
                elseif norm==2
                    deltaT=factores_DCT_discreta.*blo;
                    To=deltaT(1,1);
                    contraste=deltaT/To;
                    Lm(ii,jj,i)=deltaT(1,1);
                    contraste(1,1)=0;                  
                else
                    deltaT=factores_DCT_discreta.*blo;
                    contraste=deltaT/To_yuv(1);
                    Lm(ii,jj,i)=deltaT(1,1);
                    contraste(1,1)=0;                  
                end
            else    
                if norm==1
                    if i==2
                       W=To';
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;DUmax;0]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(2);
                       % div=DUmax;   % Para aplicar la formula sencilla CM=deltaT/deltaTmax
                    else    
                       W=To';
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;0;DVmax]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(3);                       
                       % div=DVmax;   % Para aplicar la formula sencilla CM=deltaT/deltaTmax
                    end    
                    deltaT=factores_DCT_discreta.*blo;
                    contraste=(2/YW)*deltaT/div;
                    % contraste=deltaT/div;  % Para aplicar la formula sencilla CM=deltaT/deltaTmax                  
                    Lm(ii,jj,i)=deltaT(1,1);
                    contraste(1,1)=0;
                elseif norm==2
                    To_yuv=[0;0;0];
                    for lala=1:3
                        bobo=sacasub(ss(:,:,lala),[iii jjj],[lcuan lcuan],0);
                        To_yuv(lala)=bobo(1,1)/lcuan;
                    end    
                    To=Myuv_rgb*To_yuv;
                    if i==2
                       W=To;
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;DUmax;0]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(2);
                    else    
                       W=To;
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;0;DVmax]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(3);                       
                    end    
                    deltaT=factores_DCT_discreta.*blo;
                    contraste=(2/YW)*deltaT/div;
                    Lm(ii,jj,i)=deltaT(1,1);
                    contraste(1,1)=0;            
                else
                    if i==2
                       W=To';
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;DUmax;0]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(2);
                    else    
                       W=To';
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;0;DVmax]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(3);                       
                    end    
                    deltaT=factores_DCT_discreta.*blo;
                    contraste=(2/YW)*deltaT/div;
                    Lm(ii,jj,i)=deltaT(1,1);
                    contraste(1,1)=0;                  
                end
            end
            dctf(:,:,i)=metesub(dctf(:,:,i),[iii jjj],contraste);               
        end 
    end
end    


