function MPE = distance_DCT_color(Im,ImR)

% DISTANCE_DCT_COLOR computes perceptual differences between color images
% by using Euclidean measure in the V1 cortex image representation.
%
% The V1 model used in this function consists of:
%  - Block DCT transform
%  - Divisive normalization in each (opponent) chromatic channel including 
%    frequency selectivity and masking among the DCT coefficients within 
%    each block.
% 
% The metric first computes the V1 representation of each input images
% (original and distorted), and adds the (perceived) distortion in each
% coefficient according to a 2-norm Euclidean summation.
%
% SYNTAX:
% 
%    d_MPE_DCT = distance_DCT_color(im_o,im_d)
%
%       im_o = Original RGB Image (256*256*3)
%       im_d = Distorted RGB Image (256*256*3)
%
%       d_MPE_DCT = perceptual distance
%
%       The measure can also be used for achromatic images: 
%       the program creates 256*256*3 color images by repeating the
%       luminance information in the RGB channels
%
% CITATION:
%
%  J. Malo, J. Gutierrez, J.Muoz and M. Simon. "VistaQualityTools: an image 
%  quality assessment toolbox for Matlab", 
%  http://www.uv.es/vista/vistavalencia/software.html
%
%  [Pons99] A. Pons, J.Malo, J.M.Artigas, P.Capilla 
%  Image quality metric based on multidimensional contrast perception models. 
%  Displays Journal. Vol. 20, pp. 93-110. (1999) 
%
%  [Epifanio03] I. Epifanio, J. Gutirrez and J.Malo 
%  Linear Transform for Simultaneous Diagonalization of Covariance and Perceptual 
%  Metric Matrix in Image Coding. Pattern Recognition, Vol. 36, pp. 1799-1811 (2003) 
% 

warning('off','MATLAB:dispatcher:InexactMatch')

% Dimensiones de la imagen
[Nf Nc capas] = size(Im);

if capas==1
   for capa=1:3
       Imo(:,:,capa)=Im;
       Imd(:,:,capa)=ImR;
   end
   Im=Imo;
   ImR=Imd;
   capas=3;
   clear Imo Imd
end    

% Tamanyo del bloque
N = 16;

% Transformadas color, DCT y contraste
Im = double(Im);
ImR = double(ImR);

Im_yuv = my_rgb2yuv(Im);
ImR_yuv = my_rgb2yuv(ImR);

for i = 1:capas
    dcti(:,:,i)  = dct2r(Im_yuv(:,:,i),4);
    dctiR(:,:,i) = dct2r(ImR_yuv(:,:,i),4);
end

[dct_contr, Lm] = contras3(dcti,4,1,[100 0 0],40,43);
[dct_contrR,Lm] = contras3(dctiR,4,1,[100 0 0],40,43);

% Parametros de la respuesta para cada canal
cero = 1;
exponente = [2 2 2];
tipo_alfa = 1;
[h,alpha,beta,gamm]     = constantes_respY(exponente(1),cero,tipo_alfa);
[hU,alphaU,betaU,gammU] = constantes_respU(exponente(2),cero);
[hV,alphaV,betaV,gammV] = constantes_respV(exponente(3),cero);

Lb = N;
Li = Nf;
R = [];
RR = [];

disp('  Computing DCT-normalized V1 representation...')
for cap=1:capas
    for i=1:Lb:Li
        for j=1:Lb:Li

            % Preparacion de los coeficientes de la DCT
            aa  = dct_contr(i:(i+(Lb-1)),j:(j+(Lb-1)),cap);
            aaR = dct_contrR(i:(i+(Lb-1)),j:(j+(Lb-1)),cap);            
          
            if cap==1
                [h_a,r,GR,alfa2d,beta2d]=respue5(aa,alpha,beta,gamm,h,0);
                [h_a,rR,GR,alfa2d,beta2d]=respue5(aaR,alpha,beta,gamm,h,0);
            elseif cap==2
                [h_a,r,GR,alfa2dU,beta2dU]=respue5(aa,alphaU,betaU,gammU,hU,0);
                [h_a,rR,GR,alfa2dU,beta2dU]=respue5(aaR,alphaU,betaU,gammU,hU,0);
            else
                [h_a,r,GR,alfa2dV,beta2dV]=respue5(aa,alphaV,betaV,gammV,hV,0);
                [h_a,rR,GR,alfa2dV,beta2dV]=respue5(aaR,alphaV,betaV,gammV,hV,0);
            end
            
            r(1,1) = 0;
            rR(1,1) = 0;
            
            R = [R; zigzag(r)];
            RR = [RR; zigzag(rR)];            
            
        end
    end
end
%plot(R(:) - RR(:))
MPE = sqrt( sum( ( R(:) - RR(:) ) .^ 2 ) / numel(R) );