function [MPE] = distancia_dominio_r(Im,ImR,datos)

% Calcula las distancias perceptuales despues de aplicar el modelo no
% lineal correspondiente al vector wavelet obtenido de las imagenes en
% cuestion en el espacio oponente YUV
%
% Im      -> Imagen original (RGB)
% ImR     -> Imagen distorsionada (RGB)
% datos   -> Variable con los datos de no linealidad necesarios para
%            jpeg2000_encoder
%
% USO: [MPE] = distancia_dominio_r(Im,ImR,datos)

% Datos de la respuesta no lineal
extended = datos.extended;

% Pasamos las imagenes a 'double'
Im  = double(Im);
ImR = double(ImR);

% Dimensiones de la imagen
[Nf Nc capas] = size(Im);

% Definimos los perfiles CSF (alfas) para aplicar a cada subbanda para los
% casos point-wise y extended
% Filas    -> Escala          (de mas alta frecuencia a menos)
% Columnas -> Orientacion     (horizontal, vertical, diagonal)
% Z        -> Canal cromatico (Y, U, V)

if(extended == 0 || extended == 1)
    
    % Alfas para una orientacion y todas las escalas (en columna) de mayor
    % frecuencia a menor

    % Signal Processing: Image Communication (W. Zeng et al.)

    % Y
    alfas_Y_H = [0.275783 0.837755 0.999994 1.000000 1.000000]';
    alfas_Y_V = [0.275783 0.837755 0.999994 1.000000 1.000000]';
    alfas_Y_D = [0.090078 0.701837 0.999988 1.000000 1.000000]';

    % U
    alfas_U_H = [0.089950 0.267216 0.488887 0.679829 0.812612]';
    alfas_U_V = [0.089950 0.267216 0.488887 0.679829 0.812612]';
    alfas_U_D = [0.027441 0.141965 0.348719 0.567414 0.737656]';

    % V
    alfas_V_H = [0.166647 0.375176 0.587213 0.749805 0.856065]';
    alfas_V_V = [0.166647 0.375176 0.587213 0.749805 0.856065]';
    alfas_V_D = [0.070185 0.236030 0.457826 0.655884 0.796493]';

    % Datos para la transformada Wavelet
    escalas = size(alfas_Y_H,1);
    orientaciones = 3;

    % Inicializamos la variable
    alfas = zeros(escalas,orientaciones,capas);

    % Y
    alfas(:,:,1) = [alfas_Y_H alfas_Y_V alfas_Y_D];

    % U
    alfas(:,:,2) = [alfas_U_H alfas_U_V alfas_U_D];

    % V
    alfas(:,:,3) = [alfas_V_H alfas_V_V alfas_V_D];

    % Para el caso de la Normalizacion Divisiva cargamos tambien el kernel
else

    % Extraemos los datos de interes
    escalas = datos.escalas;
    sigma_e = datos.sigma_e;
    sigma_o = datos.sigma_o;
    sigma_xy = datos.sigma_xy;
    umbral = datos.umbral;
    Ao_y = datos.Ao_y;
    Ao_uv = datos.Ao_uv;
    sigma_hv_y = datos.sigma_hv_y;
    sigma_hv_uv = datos.sigma_hv_uv;
    sigma_d_y = datos.sigma_d_y;
    sigma_d_uv = datos.sigma_d_uv;
    theta = datos.theta;

    alfas = generacion_alfas_v2([Ao_y Ao_uv],[sigma_hv_y sigma_hv_uv],...
        [sigma_d_y sigma_d_uv],theta,escalas);

    % Convertimos los 'double' en 'string' para cargar la matriz
    se = num2str(sigma_e);
    so = num2str(sigma_o);
    sxy = num2str(sigma_xy);
    tamanyo = num2str(Nf);
    s_umbral = num2str(umbral);

    indice = findstr(se,'.');
    se(indice) = '_';

    indice = findstr(so,'.');
    so(indice) = '_';

    indice = findstr(sxy,'.');
    sxy(indice) = '_';

    % Cargamos la matriz H (h_sparse)
    cd '/home/masimo/Ajuste Medida Distorsion/Kernels/Optimizados/H256/';
    load(['h_' tamanyo '_se_' se '_so_' so '_sxy_' sxy '_umbral_' s_umbral]);

end

% Espacio oponente de colores
% YUV
Imyuv  = my_rgb2yuv(Im);
ImRyuv = my_rgb2yuv(ImR);

% Transformada Wavelet Ortogonal
pyr = zeros(Nf*Nc,capas);
pyrR = pyr;

for capa = 1:capas
    [pyr(:,capa) ind] = buildWpyr(Imyuv(:,:,capa),escalas);
    [pyrR(:,capa) ind] = buildWpyr(ImRyuv(:,:,capa),escalas);
end

% Calculo de la respuesta
if(extended == 1)

    g1 = datos.g1;
    g2 = datos.g2;
    beta = datos.beta;
    tam_k = datos.tam_k;

    r  = respuesta_no_lineal(pyr,ind,alfas,extended,g1,g2,beta,tam_k);
    rR = respuesta_no_lineal(pyrR,ind,alfas,extended,g1,g2,beta,tam_k);

elseif(extended == 0)

    g1 = datos.g1;

    r  = respuesta_no_lineal(pyr,ind,alfas,extended,g1);
    rR = respuesta_no_lineal(pyrR,ind,alfas,extended,g1);

else

    g1 = datos.g1;
    beta = datos.beta;
    
    r = respuesta_no_lineal(pyr,ind,alfas,extended,g1,beta,h_sparse);
    rR = respuesta_no_lineal(pyrR,ind,alfas,extended,g1,beta,h_sparse);

end

% Eliminamos la continua de la respuesta
tam_cont = Nf *Nc / 2^(2*escalas);

r  = r(1:end-tam_cont,:);
rR = rR(1:end-tam_cont,:);

% Calculo de las distancias perceptuales
dr = abs(r - rR);

% Calculo del error perceptual como la suma cuadratica de las distancias
% perceptuales
MPE = sqrt(sum(sum(dr.^2))/(3*(Nf*Nc-tam_cont)));