function r = respuesta_no_lineal(pyr,ind,alfas,extended,exp1,exp2,beta,h_sparse,tam_k,inversa)

% Calcula la respuesta no lineal de una transformada wavelet.
% Puede obtener los modelos Point-Wise, Point-Wise Extended y Divisive
% Normalization.
%
% 'Alfas' debe tener la siguiente estructura
% Filas    (1) -> Escala          (de mas alta frecuencia a menos)
% Columnas (2) -> Orientacion     (horizontal, vertical, diagonal)
% Z        (3) -> Canal cromatico (Y, U, V)
%
% La segunda dimension del vector 'pyr' debe coincidir con la tercera de
% las 'alfas'.
%
% USOS:
%
% Point-Wise:
% r   = respuesta_no_lineal(pyr,ind,alfas,extended,exp1)
% pyr = respuesta_no_lineal(r,ind,alfas,extended,exp1,inversa)
%
% Point-Wise Extended:
% r   = respuesta_no_lineal(pyr,ind,alfas,extended,exp1,exp2,beta,tam_k)
% pyr = respuesta_no_lineal(r,ind,alfas,extended,exp1,exp2,beta,tam_k,inversa)
%
% Divisive Normalization:
% r   = respuesta_no_lineal(pyr,ind,alfas,extended,exp1,beta,h_sparse)
% pyr = respuesta_no_lineal(r,ind,alfas,extended,exp1,beta,h_sparse,inversa)

switch(extended)
    
    case 0,
        if(nargin == 5)
            inversa = 0;
        elseif(nargin == 6)
            inversa = 1;
        end
        
    case 1,
        if(nargin == 8)
            tam_k = h_sparse;
            inversa = 0;
        elseif(nargin == 9)
            tam_k = h_sparse;
            inversa = 1;
        end
        
    case 2,
        if(nargin == 7)
            inversa = 0;
            h_sparse = beta;
            beta = exp2;
        elseif(nargin == 8)
            inversa = 1;
            h_sparse = beta;
            beta = exp2;
        end

end

% if(size(pyr,2) ~= size(alfas,3))
%     error('El numero de canales cromaticos no coincide.');
% end

% Datos
[escalas orientaciones canales] = size(alfas);
capas = size(pyr,2);

% Nf = 2 * ind(1,1);
% Nc = 2 * ind(1,2);


% Vector de respuestas
r = zeros(size(pyr));


% Caso extended
if(extended == 1)

    for capa = 1:capas % Y, U, V
        for escala = 1:escalas % De mas alta frecuencia a mas baja
            for orientacion = 1:orientaciones % H V D

                % Calculamos la subbanda a procesar segun la escala y la
                % orientacion donde nos encontramos
                banda = orientaciones * (escala - 1) + orientacion;

                % Calculamos los indices para dicha subbanda
                indices = pyrBandIndices(ind,banda);

                % Obtenemos los coeficientes de la banda ordenados en forma
                % espacial
                pyr_esp = pyrBand(pyr(:,capa),ind,banda);

                if(inversa)

                    % Calculamos la inversa de la respuesta para dicha subbanda
                    % y la almacenamos en el vector pyr
                    r_esp = normalizacion_daly(pyr_esp,alfas(escala,orientacion,capa),beta,exp1,exp2,tam_k,inversa);
                    r(indices,capa) = r_esp(:);
                    %                     r(indices,capa) = normal_daly(pyr(indices,capa),alfas(escala,orientacion,capa),beta,exp1,exp2,tam_k,1);

                else

                    % Calculamos la respuesta para dicha subbanda y la almacenamos
                    % en el vector de respuesta
                    r_esp = normalizacion_daly(pyr_esp,alfas(escala,orientacion,capa),beta,exp1,exp2,tam_k);
                    r(indices,capa) = r_esp(:);
                    %                     r(indices,capa) = filter_daly(pyr(indices,capa),alfas(escala,orientacion,capa),beta,exp1,exp2,tam_k);

                end

            end
        end
    end


    % Caso Point-Wise (sin extended)
elseif(extended == 0)

    for capa = 1:capas % Y, U, V
        for escala = 1:escalas % De mas alta frecuencia a mas baja
            for orientacion = 1:orientaciones % H V D

                % Calculamos la subbanda a procesar segun la escala y la
                % orientacion donde nos encontramos
                banda = orientaciones * (escala - 1) + orientacion;

                % Calculamos los indices para dicha subbanda
                indices = pyrBandIndices(ind,banda);

                if(inversa)

                    % Calculamos la inversa de la respuesta para dicha subbanda
                    % y la almacenamos en el vector pyr
                    r(indices,capa) = sign(pyr(indices,capa)) .* (abs(pyr(indices,capa)) .^ (1/exp1)) / alfas(escala,orientacion,capa);

                else

                    % Calculamos la respuesta para dicha subbanda y la almacenamos
                    % en el vector de respuesta
                    r(indices,capa) = sign(pyr(indices,capa)) .* (abs(pyr(indices,capa) .* alfas(escala,orientacion,capa))) .^ exp1;

                end

            end
        end
    end

    % Caso Divisive Normalization
else

%     % Cargamos el kernel empleado
%     cd '/home/masimo/Ajuste Medida Distorsion/Kernels/Normales/';
%     load kernel_256_4esc % (h_sparse)

    for capa = 1:capas % Y, U, V

        if(~inversa)
            
            % Creamos un vector 'a' similar a 'r' sin la componente
            % de continua pero de una capa
            a = zeros(size(pyr,1)-prod(ind(end,:)),1);
            
            for escala = 1:escalas % De mas alta frecuencia a mas baja
                for orientacion = 1:orientaciones % H V D

                    % Calculamos la subbanda a procesar segun la escala y la
                    % orientacion donde nos encontramos
                    banda = orientaciones * (escala - 1) + orientacion;

                    % Calculamos los indices para dicha subbanda
                    indices = pyrBandIndices(ind,banda);

                    % Aplicamos la CSF y la no linealidad a cada subbanda                    
                    a(indices) = (abs(pyr(indices,capa) .* alfas(escala,orientacion,capa))) .^ exp1;

                end
            end

            % En cada capa calculamos la Normalizacion Divisiva
            r(1:size(a,1),capa) = a ./ ( beta + h_sparse * a );
            r(:,capa) = sign(pyr(:,capa)) .* abs(r(:,capa));

        else
            
            % Invertimos la Normalizacion Divisiva, eliminando la no
            % linealidad y recuperando el signo original del vector wavelet
            a_rec = inv_normal_div(pyr(:,capa),exp1,beta,h_sparse,100);

            for escala = 1:escalas % De mas alta frecuencia a mas baja
                for orientacion = 1:orientaciones % H V D
                    
                    % Calculamos la subbanda a procesar segun la escala y la
                    % orientacion donde nos encontramos
                    banda = orientaciones * (escala - 1) + orientacion;

                    % Calculamos los indices para dicha subbanda
                    indices = pyrBandIndices(ind,banda);

                    % Aplicamos la CSF y la no linealidad a cada subbanda
                    r(indices,capa) = (a_rec(indices) ./ alfas(escala,orientacion,capa));
                    
                    % Si los valores de alfa son nulos en r tendremos
                    % valores NaN. Los sustituiremos por 0.
                    r(isnan(r)) = 0;
                    
                end
            end
            
        end

    end
end