% function [Id,PSF] = apply_degradation_2(Im,Type,varargin);
%
%  APPLY_DEGRADATON_2 degrades the input image according to some
%  degradation model. The available degradation models include:
%
%     * 'wt_gn'      : White Gaussian Noise
%     * 'col_gn'     : Colored Gaussian Noise
%     * 'blur_wt_gn' : Blur plus white Gaussian Noise
%     * 'blur_col_gn': Blur plus colored Gaussian Noise
%     * 'jpeg'       : JPEG degradation due to compression
%     * 'salt_pepper': Salt and Pepper Noise.
%
%  The number of input arguments depends on the degradation model
%  They have to be included in the described order:
%
%     * 'wt_gn'      : White Gaussian Noise
%                      Needs (1) the variance of the noise
%     * 'col_gn'     : Colored Gaussian Noise
%                      Needs (1) the frequency band limits of the noise 
%                                [f_min f_max] expresed in normalized units
%                                (in the range [0,1], where 1 is the
%                                Nyquist frequency).
%                            (2) The variance of the noise
%                       
%     * 'blur_wt_gn' : Blur plus white Gaussian Noise
%                      Needs (1) the cut-off frequency of the blurring
%                                operator (in normalized units).
%                            (2) the minimum value of the blurring operator 
%                                (in the range [0,1]).
%                                In order to preserve some information of 
%                                high frequency components, the user can 
%                                introduce a minimum value for the amplitude 
%                                of the blurring filter (maximum attenuation 
%                                factor).
%                                Value 0 means that the Gaussian low-pass
%                                operator will not be modified.
%                            (3) The variance of the noise.
% 
%     * 'blur_col_gn': Blur plus colored Gaussian Noise
%                      Needs (1) the frequency band limits of the noise 
%                                [f_min f_max] expresed in normalized units
%                                (in the range [0,1], where 1 is the
%                                Nyquist frequency).
%                            (2) the cut-off frequency of the blurring
%                                operator (in normalized units).
%                            (3) the minimum value of the blurring operator 
%                                (in the range [0,1]).
%                                In order to preserve some information of 
%                                high frequency components, the user can 
%                                introduce a minimum value for the amplitude 
%                                of the blurring filter (maximum attenuation 
%                                factor).
%                                Value 0 means that the Gaussian low-pass
%                                operator will not be modified.
%                            (4) The variance of the noise.
%
%     * 'jpeg'       : JPEG degradation due to compression
%                      Needs (1) the quality parameter (see imwrite)
%
%     * 'salt_pepper': Salt and Pepper Noise.
%                      Needs (1) noise density (see imnoise)
%
%   OUTPUT:
%   Id : degraded image
%   PSF: the Point Spread Function of the blurring (this can be just 
%        a delta function if there is no blurring).
% 
%   SYNTAX:
%
%   [degraded,PSF] = apply_degradation_2(Im,'Degradat_Model',Param1,Param2,...);
%   
% J. Gutierrez, V. Laparra, G. Camps and J. Malo. "VistaRestoreTools: an image 
% restoration toolbox for Matlab", http://www.uv.es/vista/vistavalencia/software.html
%
 
function [degraded,PSF] = apply_degradation(Im,type,varargin);

warning('off','MATLAB:dispatcher:InexactMatch')
if (strcmpi(type,'wt_gn'))

   % This is just to use our function degrade 

   % Cutoff frequency of the blurring operator (in this case no blur)
   fc = 3;
   Val_min = 0.001;

   % Frecuency band of the noise 
   % In this case: white noise
   lim_frec = [0 3];

   % Noise variance
   variance = varargin{1};
   
   [degraded,PSF,H,noise,Blurred]=degrade(Im,fc,Val_min,lim_frec,variance);     

elseif  (strcmpi(type,'col_gn'))
    
   % This is just to use our function degrade 

   % Cutoff frequency of the blurring operator (in this case no blur)
   fc = 3;
   Val_min = 0.001;

   % Frecuency band of the noise 
   lim_frec = varargin{1};

   % Noise variance
   variance = varargin{2};
   
   [degraded,PSF,H,noise,Blurred]=degrade(Im,fc,Val_min,lim_frec,variance);     
    
   
elseif  (strcmpi(type,'blur_wt_gn'))

   % This is just to use our function degrade 

   % Cutoff frequency of the blurring operator
   fc = varargin{1};
   Val_min = varargin{2};

   % Frecuency band of the noise 
   % In this case: white noise
   lim_frec = [0 3];

   % Noise variance
   variance = varargin{3};

   [degraded,PSF,H,noise,Blurred]=degrade(Im,fc,Val_min,lim_frec,variance);
   
elseif  (strcmpi(type,'blur_col_gn'))

   % This is just to use our function degrade 

   % Cutoff frequency of the blurring operator
   fc = varargin{1};
   Val_min = varargin{2};

   % Frecuency band of the noise 
   % In this case: white noise
   lim_frec = varargin{3};

   % Noise variance
   variance = varargin{4};

   [degraded,PSF,H,noise,Blurred]=degrade(Im,fc,Val_min,lim_frec,variance);   
   
elseif  (strcmpi(type,'jpeg'))

   PSF = zeros(11);
   PSF(6,6)=1;
   
   imwrite(Im/255,'tmp.jpg','Quality',varargin{1});
   degraded = imread('tmp.jpg');
   
elseif  (strcmpi(type,'salt_pepper'))

   PSF = zeros(11);
   PSF(6,6)=1;
   degraded = imnoise(Im/255,'salt & pepper', varargin{1});
   degraded = degraded*255;
else
   error('The requested degradation is not implemented');
end

degraded = double(degraded);
