%
%   DEGRADE crea una imagen degradada test usando una PSF espacialmente
%   invariante h (filtro pasa-baja gaussiano con frecuencia de corte normalizada 
%   fc, 0<fc<1 -siendo 1 la frecuencia de Nyquist-).
%   Ademas, aade un ruido coloreado isotropo 
%   en la banda 0<[f1,f2]<1 con la varianza que se le pida.
%
%   NOTA: Con objeto de permitir la permanencia de seales de frecuencia mayor 
%   que la de corte, introducimos un valor 'val' en la respuesta en
%   frecuencia.
%   Como normalizamos la PSF para conservar la componente de continua de la
%   seal, el valor final de la respuesta en frecuencia para altas
%   frecuencias no es 'val', sino ligeramente superior.
%   En cualquier caso, el programa devuelve H (respuesta en frecuencia del
%   filtro de emborronamiento), y el usuario puede controlar este valor.
%   Seleccionando val=0 no se introduce ninguna modificacion en la
%   normalizacion.
%
%    La imagen degradada resultante es:
%    Id = conv2(Im,h) + ruido;
%
%
%    PARAMETROS:
%
%    Im: imagen a degradar
%    fcorte: frecuencia de corte de la PSF
%    val: parametro para controlar el valor de la respuesta en frecuencia 
%         por encima de la frecuencia de corte
%    [f1 f2]: Vector con las frecuencias de la banda del ruido coloreado.
%    vari: varianza del ruido aditivo.
%
%    DEVUELVE:
%    Id:  imagen degradada    
%    PSF: filtro emborronamiento utilizado
%    H: respuesta en frecuencia H=abs(freqz2(PSF));
%    noise: ruido aadido a la imagen
%    blurred: imagen emborronada antes de aadir el ruido
%    
%    USO:
%    [Imdegradada,PSF,H,noise,Blurred]=degrada2(imagen,fcorte,Val_min,[f1 f2],varianza);


function [Id,PSF,H,noise,Blurred]=degrade(Im,fc,Val_min,lim_frec,vari);

N=size(Im);

% Extension en pixels del filtro
Numpf = 11;

[f1,f2] = freqspace(Numpf,'meshgrid');
Hd = ones(Numpf);
r = sqrt(f1.^2 + f2.^2);
Hd(r>fc) = Val_min;
win = fspecial('gaussian',Numpf,2);
win = win ./ max(win(:));  % Make the maximum window value be 1.
PSF = fwind2(Hd,win);
PSF=PSF/sum(sum(PSF));
H=abs(freqz2(PSF));


% Convolucion de la imagen con el filtro de emborronamiento
% Primero amplia la imagen antes de convolucionar para evitar efectos de
% borde...
Im2 = ampliaconborde(Im,10);

Im2 = conv2(Im2,PSF,'same');

Blurred = Im2(11:11+N(1)-1,11:11+N(2)-1);

[f1,f2] = freqspace(N,'meshgrid');
r = sqrt(f1.^2 + f2.^2);
Hruido=(r>=lim_frec(1))&(r<=lim_frec(2));
tfruido=Hruido.*exp(sqrt(-1)*2*pi*rand(N));
ruido=real(ifft2(fftshift(tfruido)));
varruido=var(ruido(:));

noise = sqrt(vari)*ruido/sqrt(varruido);

Id = Blurred + noise;