% 
% VistaQualityTools1.0
% IMAGE QUALITY DEMO
% 
% This script implements a set of controlled degradations on a particular
% image and applies a number of distortion measures (RMSE, PSNR, d_DCT,
% d_wav).
% 
% The considered degradations are:
%
%    * Gaussian blur (of tunable cut-off frequency) plus Gaussian colored
%      noise (of tunable frequency band and variance).
%      With appropriate parameters you can generate degradations ranging
%      from pure blurring to pure denoising (including white or colored noise).
%
%    * JPEG degradation: an example of non-Gaussian noise
%
%    * Salt & Pepper: another non-Gaussian noise
%
% The parameters will be set to obtain similar MSE (and PSNR) but quite different
% visual appearance. Order them according to your opinion and compare with 
% the ranking given by the algorithms...
% 

%%%%%%%%%%%%%%%%%%
%                %
% Original Image %
%                %
%%%%%%%%%%%%%%%%%%

load barbara256   % The image variable is named Im

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                        %
% BLUR + WHITE OR COLORED GAUSSIAN NOISE %
%                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% EXAMPLES
% --------
%
% (1) No blur + Gaussian white noise
% (2) No blur + Gaussian colored noise
% (3) Moderate blur + Gaussian colored noise
% (4) Severe blur + Gaussian colored noise

% (1) No blur + Gaussian white noise

deg_model='wt_gn';
variance=205;

[Imd,PSF] = apply_degradation_2(Im,deg_model,variance);
figure,colormap gray,imagesc(Imd);axis square,
title(['Gaussian White Noise  \sigma^2=',num2str(variance)])
RMSE=sqrt(sum(sum((Im-Imd).^2)))/256;
PSNR=psnr(Im,Imd);
d_MPE_wav = distance_wav_color(Im,Imd);
d_MPE_dct = distance_DCT_color(Im,Imd);
xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   RMSE=',num2str(round(10*RMSE)/10)],['d_{wav}=',num2str(round(1000*d_MPE_wav)/1000),'  d_{DCT}=',num2str(round(10000*d_MPE_dct)/10000)]))


%xlabel(['RMSE^{1/2}=',num2str(round(rmse*10)/10),'  d_{wav}=',num2str(d_MPE_wav(1))])

% (2) No blur + Gaussian colored noise

deg_model='col_gn';
band_limits_noise=[0.25 0.5];
variance=205;
[Imd,PSF] = apply_degradation_2(Im,deg_model,band_limits_noise,variance);
figure,colormap gray,imagesc(Imd);axis square,
title(['Gaussian Colored Noise  \sigma^2=',num2str(variance)])
RMSE=sqrt(sum(sum((Im-Imd).^2)))/256;
PSNR=psnr(Im,Imd);
d_MPE_wav = distance_wav_color(Im,Imd);
d_MPE_dct = distance_DCT_color(Im,Imd);
xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   RMSE=',num2str(round(10*RMSE)/10)],['d_{wav}=',num2str(round(1000*d_MPE_wav)/1000),'  d_{DCT}=',num2str(round(10000*d_MPE_dct)/10000)]))

% (3) Moderate blur + Gaussian colored noise

deg_model='blur_col_gn';
cut_off_blur=0.6;
min_val_blur=0.001;
band_limits_noise=[0.25 0.5];
variance=160;
[Imd,PSF] = apply_degradation_2(Im,deg_model,cut_off_blur,min_val_blur,band_limits_noise,variance);
figure,colormap gray,imagesc(Imd);axis square,
title(['Moderate blur  f_{cut}=',num2str(cut_off_blur), ' + Gaussian Colored Noise  \sigma^2=',num2str(variance)])
RMSE=sqrt(sum(sum((Im-Imd).^2)))/256;
PSNR=psnr(Im,Imd);
d_MPE_wav = distance_wav_color(Im,Imd);
d_MPE_dct = distance_DCT_color(Im,Imd);
xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   RMSE=',num2str(round(10*RMSE)/10)],['d_{wav}=',num2str(round(1000*d_MPE_wav)/1000),'  d_{DCT}=',num2str(round(10000*d_MPE_dct)/10000)]))


% (4) Severe blur + Gaussian colored noise

deg_model='blur_col_gn';
cut_off_blur=0.4;
min_val_blur=0.001;
band_limits_noise=[0.25 0.5];
variance=17;
[Imd,PSF] = apply_degradation_2(Im,deg_model,cut_off_blur,min_val_blur,band_limits_noise,variance);
figure,colormap gray,imagesc(Imd);axis square,
title(['Severe blur  f_{cut}=',num2str(cut_off_blur), ' + Gaussian Colored Noise  \sigma^2=',num2str(variance)])
RMSE=sqrt(sum(sum((Im-Imd).^2)))/256;
PSNR=psnr(Im,Imd);
d_MPE_wav = distance_wav_color(Im,Imd);
d_MPE_dct = distance_DCT_color(Im,Imd);
xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   RMSE=',num2str(round(10*RMSE)/10)],['d_{wav}=',num2str(round(1000*d_MPE_wav)/1000),'  d_{DCT}=',num2str(round(10000*d_MPE_dct)/10000)]))


%%%%%%%%%%%%%%%%%%%%
%                  %
% JPEG DEGRADATION %
%                  %
%%%%%%%%%%%%%%%%%%%%

% In JPEG the quality (and hence the compresion ratio) is controlled by the
% parameter 'Quality' in the range [0, 100].
%
% In order to obtain challenging degradations for denoising algorithms, set
% this value in the range [1, 15]

deg_model='jpeg';
Quality = 7;
[Imd,PSF] = apply_degradation_2(Im,deg_model,Quality);
figure,colormap gray,
imagesc(Imd);axis square,title(['JPEG Image with quality factor ' num2str(Quality)])
RMSE=sqrt(sum(sum((Im-Imd).^2)))/256;
PSNR=psnr(Im,Imd);
d_MPE_wav = distance_wav_color(Im,Imd);
d_MPE_dct = distance_DCT_color(Im,Imd);
xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   RMSE=',num2str(round(10*RMSE)/10)],['d_{wav}=',num2str(round(1000*d_MPE_wav)/1000),'  d_{DCT}=',num2str(round(10000*d_MPE_dct)/10000)]))


%%%%%%%%%%%%%%%%%
%               %
% SALT & PEPPER %
%               %
%%%%%%%%%%%%%%%%%

% Salt & Pepper noise consists on changing arbitrary pixels by the minimum
% and maximum possible values of the signal (e.g., 0 and 255 in 8-bit images)
%
% This function uses 'imnoise.m': the amount of corrupted pixels is given
% by the parameter D, which ranges in the interval [0,1]. The number of
% affected pixels is approximately D*num_pixels.
%

deg_model='salt_pepper';
D=0.011;
[Imd,PSF] = apply_degradation_2(Im,deg_model,D);
figure,colormap gray,
imagesc(Imd);axis square,title(['Salt & Pepper noise. Percentage affected=' num2str(D*100) '%'])
RMSE=sqrt(sum(sum((Im-Imd).^2)))/256;
PSNR=psnr(Im,Imd);
d_MPE_wav = distance_wav_color(Im,Imd);
d_MPE_dct = distance_DCT_color(Im,Imd);
xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   RMSE=',num2str(round(10*RMSE)/10)],['d_{wav}=',num2str(round(1000*d_MPE_wav)/1000),'  d_{DCT}=',num2str(round(10000*d_MPE_dct)/10000)]))

